import _extends from "@babel/runtime/helpers/esm/extends";
import optimizelySDK from '@optimizely/optimizely-sdk';
import { updateIntegrations } from './integrations/handler.js';
var DEFAULT_DATAFILE_OPTIONS = {
  autoUpdate: true,
  updateInterval: 5 * 60 * 1000 // 5 minutes
};
var DEFAULT_EVENTS_OPTIONS = {
  eventBatchSize: 10,
  eventFlushInterval: 1000
};
var DEFAULT_TIMEOUT = 500;
var enums = optimizelySDK.enums;
var LOGGER_LEVEL = process.env.NODE_ENV === 'production' ? enums.error : enums.info;
var OptimizelyAdapter = /*#__PURE__*/function () {
  /**
   * @constructor
   * @param {object} param
   * @param {object} param.optimizely OptimizelyInstance returned by createOptimizelyInstance method
   * @param {string=} param.userId
   * @param {object} param.activeIntegrations segment activated by default
   * @param {boolean} param.hasUserConsents
   */
  function OptimizelyAdapter(_ref) {
    var optimizely = _ref.optimizely,
      userId = _ref.userId,
      _ref$activeIntegratio = _ref.activeIntegrations,
      activeIntegrations = _ref$activeIntegratio === void 0 ? {
        segment: true
      } : _ref$activeIntegratio,
      hasUserConsents = _ref.hasUserConsents,
      _ref$applicationAttri = _ref.applicationAttributes,
      applicationAttributes = _ref$applicationAttri === void 0 ? {} : _ref$applicationAttri;
    if (!optimizely) {
      throw new Error('optimizely instance is mandatory to use OptimizelyAdapter');
    }
    this._optimizely = optimizely;
    this._userId = userId == null ? void 0 : userId.toString();
    this._activeIntegrations = activeIntegrations;
    this._applicationAttributes = applicationAttributes;
    this.updateConsents({
      hasUserConsents: hasUserConsents
    });
  }

  /**
   * @param {object} param
   * @param {string} param.sdkKey
   * @param {object=} param.datafile
   * @param {object} param.options datafile options https://docs.developers.optimizely.com/full-stack/docs/initialize-sdk-javascript-node
   * @param {object} param.optimizely test purposes only, optimizely sdk
   * @param {function} param.eventDispatcher https://docs.developers.optimizely.com/full-stack/docs/configure-event-dispatcher-javascript-node
   */
  OptimizelyAdapter.createOptimizelyInstance = function createOptimizelyInstance(_ref2) {
    var _window$__INITIAL_CON;
    var optionParameter = _ref2.options,
      sdkKey = _ref2.sdkKey,
      datafile = _ref2.datafile,
      _ref2$optimizely = _ref2.optimizely,
      optimizely = _ref2$optimizely === void 0 ? optimizelySDK : _ref2$optimizely,
      _ref2$eventDispatcher = _ref2.eventDispatcher,
      eventDispatcher = _ref2$eventDispatcher === void 0 ? optimizelySDK.eventDispatcher : _ref2$eventDispatcher;
    var options = _extends({}, DEFAULT_DATAFILE_OPTIONS, optionParameter);
    optimizely.setLogLevel(LOGGER_LEVEL);
    optimizely.setLogger(optimizely.logging.createLogger());
    if (!datafile && typeof window !== 'undefined' && (_window$__INITIAL_CON = window.__INITIAL_CONTEXT_VALUE__) != null && _window$__INITIAL_CON.pde) {
      datafile = window.__INITIAL_CONTEXT_VALUE__.pde;
      sdkKey = undefined;
    }
    var isServer = typeof window === 'undefined';
    var optimizelyInstance = optimizely.createInstance(_extends({
      sdkKey: sdkKey,
      datafileOptions: options,
      datafile: datafile,
      eventDispatcher: eventDispatcher
    }, DEFAULT_EVENTS_OPTIONS, {
      defaultDecideOptions: isServer ? [optimizely.OptimizelyDecideOption.DISABLE_DECISION_EVENT] : []
    }));
    return optimizelyInstance;
  }

  /**
   * @param {object} param
   * @param {object} param.attributes
   * @return {string[]}
   */;
  var _proto = OptimizelyAdapter.prototype;
  _proto.getEnabledFeatures = function getEnabledFeatures(_ref3) {
    var attributes = _ref3.attributes;
    return this._optimizely.getEnabledFeatures(this._userId, attributes);
  }

  /**
   * returns the datafile, useful in a server/client scenario
   */;
  _proto.getInitialData = function getInitialData() {
    var datafile = null;
    try {
      datafile = this.getOptimizelyConfig().getDatafile();
    } catch (error) {
      // eslint-disable-next-line no-console
      console.error(error);
    }
    return datafile;
  };
  _proto.getOptimizelyConfig = function getOptimizelyConfig() {
    return this._optimizely.getOptimizelyConfig();
  }

  /**
   * Activates an A/B test for the specified user to start an experiment: determines whether they qualify for the experiment,
   * buckets a qualified user into a variation, and sends an impression event to Optimizely.
   * More info: https://docs.developers.optimizely.com/full-stack/docs/activate-javascript-node
   * @param {Object} params
   * @param {string} params.name
   * @param {object} [params.attributes]
   * @returns {string=} variation name
   */;
  _proto.activateExperiment = function activateExperiment(_ref4) {
    var name = _ref4.name,
      attributes = _ref4.attributes;
    if (!this._hasUserConsents) return null;
    return this._optimizely.activate(name, this._userId, _extends({}, this._applicationAttributes, attributes));
  }

  /**
   * @param {Object} params
   * @param {string} params.name
   * @param {object} [params.attributes]
   * @returns {object} decision
   */;
  _proto.decide = function decide(_ref5) {
    var name = _ref5.name,
      attributes = _ref5.attributes,
      isEventDisabled = _ref5.isEventDisabled;
    var user = this._optimizely.createUserContext(this._userId, _extends({}, this._applicationAttributes, attributes));
    return user.decide(name, !this._hasUserConsents || isEventDisabled ? [optimizelySDK.OptimizelyDecideOption.DISABLE_DECISION_EVENT] : undefined);
  }

  /**
   * @param {Object} params
   * @param {function} params.onDecide
   * @returns {number} notificationId
   */;
  _proto.addDecideListener = function addDecideListener(_ref6) {
    var onDecide = _ref6.onDecide;
    return this._optimizely.notificationCenter.addNotificationListener(enums.NOTIFICATION_TYPES.DECISION, onDecide);
  }

  /**
   * @param {Object} params
   * @param {number} params.notificationId
   */;
  _proto.removeNotificationListener = function removeNotificationListener(_ref7) {
    var notificationId = _ref7.notificationId;
    this._optimizely.notificationCenter.removeNotificationListener(notificationId);
  }

  /**
   * Gets the variation without tracking the impression
   * @param {Object} params
   * @param {string} params.name
   * @param {object} [params.attributes]
   * @returns {string=} variation name
   */;
  _proto.getVariation = function getVariation(_ref8) {
    var name = _ref8.name,
      attributes = _ref8.attributes;
    if (!this._hasUserConsents) return null;
    return this._optimizely.getVariation(name, this._userId, _extends({}, this._applicationAttributes, attributes));
  };
  _proto.onReady = function onReady() {
    var _this = this;
    return this._optimizely.onReady({
      timeout: DEFAULT_TIMEOUT
    }).then(function () {
      return _this._optimizely;
    });
  }

  /**
   * Checks if a feature flag is active or not
   * @param {object} param
   * @param {string} param.featureKey
   * @parma {object=} param.attributes
   * @returns {isActive: boolean, linkedExperiments: number[]}
   */;
  _proto.isFeatureEnabled = function isFeatureEnabled(_ref9) {
    var _this$getOptimizelyCo, _this$getOptimizelyCo2;
    var featureKey = _ref9.featureKey,
      attributes = _ref9.attributes;
    var experimentsMap = ((_this$getOptimizelyCo = this.getOptimizelyConfig()) == null ? void 0 : (_this$getOptimizelyCo2 = _this$getOptimizelyCo.featuresMap[featureKey]) == null ? void 0 : _this$getOptimizelyCo2.experimentsMap) || {};
    var linkedExperimentNames = Object.keys(experimentsMap);

    // check for user consents only if featureKey is a feature that belongs to a feature test or if a userId is available
    if (linkedExperimentNames.length > 0 && !this._hasUserConsents || !this._userId) {
      return {
        isActive: false,
        linkedExperiments: []
      };
    }
    return {
      isActive: this._optimizely.isFeatureEnabled(featureKey, this._userId, _extends({}, this._applicationAttributes, attributes)),
      linkedExperiments: linkedExperimentNames
    };
  }

  /**
   * Return all variables of a given feature
   * @param {object} param
   * @param {string} param.featureKey
   * @parma {object=} param.attributes
   * @returns {object}
   */;
  _proto.getAllFeatureVariables = function getAllFeatureVariables(_ref10) {
    var featureKey = _ref10.featureKey,
      attributes = _ref10.attributes;
    return this._optimizely.getAllFeatureVariables(featureKey, this._userId, attributes);
  };
  _proto.updateConsents = function updateConsents(_ref11) {
    var hasUserConsents = _ref11.hasUserConsents;
    this._hasUserConsents = hasUserConsents;
    updateIntegrations({
      hasUserConsents: this._hasUserConsents,
      activeIntegrations: this._activeIntegrations,
      optimizelyInstance: this._optimizely
    });
  };
  return OptimizelyAdapter;
}();
export { OptimizelyAdapter as default };