import { getPlatformStrategy } from './hooks/common/platformStrategies.js';

/**
 * Function to call decide
 * @param {object} pde
 * @return {function}
 */
export default function getDecision(pde) {
  if (!pde) {
    throw new Error('[sui-pde: useDecision] sui-pde provider is required to work');
  }
  var decide = function decide(name, _temp) {
    var _ref = _temp === void 0 ? {} : _temp,
      attributes = _ref.attributes,
      trackExperimentViewed = _ref.trackExperimentViewed,
      isEventDisabled = _ref.isEventDisabled,
      queryString = _ref.queryString,
      adapterId = _ref.adapterId;
    try {
      var strategy = getPlatformStrategy({
        customTrackExperimentViewed: trackExperimentViewed
      });
      var forced = strategy.getForcedValue({
        key: name,
        queryString: queryString
      });
      if (forced) {
        if (['on', 'off'].includes(forced)) {
          return {
            enabled: forced === 'on',
            flagKey: name
          };
        }
        return {
          enabled: true,
          flagKey: name,
          variationKey: forced
        };
      }
      var notificationId = pde.addDecideListener({
        onDecide: function onDecide(_ref2) {
          var type = _ref2.type,
            decision = _ref2.decisionInfo;
          var ruleKey = decision.ruleKey,
            variationKey = decision.variationKey,
            decisionEventDispatched = decision.decisionEventDispatched;
          if (type === 'flag' && decisionEventDispatched) {
            strategy.trackExperiment({
              variationName: variationKey,
              experimentName: ruleKey
            });
          }
        }
      });
      var data = strategy.decide({
        pde: pde,
        name: name,
        attributes: attributes,
        adapterId: adapterId,
        isEventDisabled: isEventDisabled
      });
      pde.removeNotificationListener({
        notificationId: notificationId
      });
      return data;
    } catch (error) {
      return {
        enabled: false,
        flagKey: name
      };
    }
  };
  return {
    decide: decide
  };
}