import { parseQueryString } from '@s-ui/js/lib/string/index.js';
import { trackedEventsLocalCache } from './trackedEventsLocalCache.js';
var getServerStrategy = function getServerStrategy() {
  return {
    getVariation: function getVariation(_ref) {
      var pde = _ref.pde,
        experimentName = _ref.experimentName,
        attributes = _ref.attributes,
        adapterId = _ref.adapterId;
      return pde.getVariation({
        pde: pde,
        name: experimentName,
        attributes: attributes,
        adapterId: adapterId
      });
    },
    decide: function decide(_ref2) {
      var pde = _ref2.pde,
        name = _ref2.name,
        attributes = _ref2.attributes,
        adapterId = _ref2.adapterId,
        isEventDisabled = _ref2.isEventDisabled;
      return pde.decide({
        pde: pde,
        name: name,
        attributes: attributes,
        adapterId: adapterId,
        isEventDisabled: isEventDisabled
      });
    },
    trackExperiment: function trackExperiment() {},
    getForcedValue: function getForcedValue(_ref3) {
      var key = _ref3.key,
        queryString = _ref3.queryString;
      if (!queryString) {
        return;
      }
      var queryParams = parseQueryString(queryString);
      return queryParams["suipde_" + key];
    }
  };
};
var getBrowserStrategy = function getBrowserStrategy(_ref4) {
  var customTrackExperimentViewed = _ref4.customTrackExperimentViewed,
    cache = _ref4.cache;
  return {
    getVariation: function getVariation(_ref5) {
      var pde = _ref5.pde,
        experimentName = _ref5.experimentName,
        attributes = _ref5.attributes,
        adapterId = _ref5.adapterId;
      var variationName = pde.activateExperiment({
        name: experimentName,
        attributes: attributes,
        adapterId: adapterId
      });
      return variationName;
    },
    decide: function decide(_ref6) {
      var pde = _ref6.pde,
        name = _ref6.name,
        attributes = _ref6.attributes,
        adapterId = _ref6.adapterId,
        isEventDisabled = _ref6.isEventDisabled;
      return pde.decide({
        pde: pde,
        name: name,
        attributes: attributes,
        adapterId: adapterId,
        isEventDisabled: isEventDisabled
      });
    },
    trackExperiment: function trackExperiment(_ref7) {
      var _window$analytics;
      var variationName = _ref7.variationName,
        experimentName = _ref7.experimentName;
      if (customTrackExperimentViewed) {
        return customTrackExperimentViewed({
          variationName: variationName,
          experimentName: experimentName
        });
      }

      // user is not part of the experiment
      if (!variationName) return;

      // if experiment has been already tracked
      if (cache.includes(experimentName, variationName)) return;
      if (!((_window$analytics = window.analytics) != null && _window$analytics.track)) {
        // eslint-disable-next-line no-console
        console.error("[sui-pde: useExperiment] window.analytics.track expected to exists but doesn't");
        return;
      }
      cache.push(experimentName, variationName);
      window.analytics.ready(function () {
        window.analytics.track('Experiment Viewed', {
          experimentName: experimentName,
          variationName: variationName
        });
      });
    },
    /**
     * @param {object} param
     * @param {string} key Experiment or feature flag key
     * @param {string} queryString Test purposes only
     * @returns {string|null}
     */
    getForcedValue: function getForcedValue(_ref8) {
      var key = _ref8.key,
        _ref8$queryString = _ref8.queryString,
        queryString = _ref8$queryString === void 0 ? document.location.search : _ref8$queryString;
      var queryParams = parseQueryString(queryString);
      return queryParams["suipde_" + key];
    }
  };
};

/**
 * Returns the implementation of experiment related methods depending on
 * which platform we are server/browser
 * @param {object} param
 * @param {function} [param.customTrackExperimentViewed]
 * @returns object
 */
export var getPlatformStrategy = function getPlatformStrategy(_temp) {
  var _ref9 = _temp === void 0 ? {} : _temp,
    customTrackExperimentViewed = _ref9.customTrackExperimentViewed;
  var isNode = typeof window === 'undefined';
  if (isNode) {
    return getServerStrategy();
  }
  trackedEventsLocalCache.init();
  return getBrowserStrategy({
    customTrackExperimentViewed: customTrackExperimentViewed,
    cache: trackedEventsLocalCache
  });
};