import { useMemo } from 'react';
import useDecisionCallback from './useDecisionCallback.js';

/**
 * Hook to use a feature test
 * @param {string} name
 * @param {object} param
 * @param {object} param.attributes
 * @param {function} param.trackExperimentViewed
 * @param {string} param.queryString
 * @param {string} param.adapterId Adapter id to be executed
 * @return {object}
 */
export default function useDecision(name, _temp) {
  var _ref = _temp === void 0 ? {} : _temp,
    attributes = _ref.attributes,
    trackExperimentViewed = _ref.trackExperimentViewed,
    isEventDisabled = _ref.isEventDisabled,
    queryString = _ref.queryString,
    adapterId = _ref.adapterId;
  var _useDecisionCallback = useDecisionCallback(),
    decide = _useDecisionCallback.decide;
  var data = useMemo(function () {
    return decide(name, {
      attributes: attributes,
      trackExperimentViewed: trackExperimentViewed,
      isEventDisabled: isEventDisabled,
      queryString: queryString,
      adapterId: adapterId
    });
  }, [name, attributes, trackExperimentViewed, isEventDisabled, queryString, adapterId]);
  return data;
}